@@ -48,6 +48,7 @@
 #define NAUTILUS_COMMAND	 BIN_NAUTILUS" -n --no-desktop %m"
 
 static struct gvm_configuration config;
+static DBusConnection *dbus_connection;
 static LibHalContext *hal_ctx;
 
 struct _MountPolicy {
@@ -95,9 +96,6 @@ enum {
 	AUTOPRINTER_COMMAND,
 	AUTORUN,
 	AUTORUN_PATH,
-	EJECT_COMMAND,
-	USER_MOUNT_COMMAND,
-	USER_UNMOUNT_COMMAND
 };
 
 static struct {
@@ -129,29 +127,6 @@ static struct {
 	{ GCONF_ROOT "autoprinter_command",       TYPE_STRING, &config.autoprinter_command       },
 	{ GCONF_ROOT "autorun",                   TYPE_BOOL,   &config.autorun                   },
 	{ GCONF_ROOT "autorun_path",              TYPE_STRING, &config.autorun_path              },
-	{ GCONF_ROOT "eject_command",             TYPE_STRING, &config.eject_command             },
-	{ GCONF_ROOT "mount_command",             TYPE_STRING, &config.mount_command             },
-	{ GCONF_ROOT "unmount_command",           TYPE_STRING, &config.unmount_command           },
-};
-
-/* mount commands to try in order of preference */
-static struct {
-	const char *command;  /* full command-line */
-	char *path;           /* path to mount program - run-time discovered */
-} mount_commands[] = {
-	{ NULL,                         NULL },  /* user override mount command */
-	{ "pmount-hal " PMOUNT_OPTIONS, NULL },  /* pmount-hal */
-	{ "mount " MOUNT_OPTIONS,       NULL },  /* mount */
-};
-
-/* unmount commands to try in order of preference */
-static struct {
-	const char *command;  /* full command-line */
-	char *path;           /* path to mount program - run-time discovered */
-} unmount_commands[] = {
-	{ NULL,                         NULL },  /* user override unmount command */
-	{ "pumount " PUMOUNT_OPTIONS,   NULL },  /* pumount */
-	{ "umount " UMOUNT_OPTIONS,     NULL },  /* umount */
 };
 
 static GHashTable *gvm_settings_hash = NULL;
@@ -591,14 +566,6 @@ gvm_load_config (void)
 				gconf_client_get_string (config.client, gvm_settings[i].key, NULL);
 			dbg ("setting[%d]: string: %s = %s\n", i, strrchr (gvm_settings[i].key, '/') + 1,
 			     *((char **) gvm_settings[i].var));
-			
-			if (i == USER_MOUNT_COMMAND) {
-				/* set the mount_commands default mount program */
-				mount_commands[0].command = config.mount_command;
-			} else if (i == USER_UNMOUNT_COMMAND) {
-				/* set the unmount_commands default unmount program */
-				unmount_commands[0].command = config.unmount_command;
-			}
 		} else if (gvm_settings[i].type == TYPE_BOOL) {
 			*((int *) gvm_settings[i].var) =
 				gconf_client_get_bool (config.client, gvm_settings[i].key, NULL);
@@ -640,18 +607,6 @@ gvm_config_changed (GConfClient *client __attribute__((__unused__)),
 		*((char **) gvm_settings[which].var) = g_strdup (gconf_value_get_string (value));
 		dbg ("setting changed: string: %s = %s\n", strrchr (gvm_settings[which].key, '/') + 1,
 		     *((char **) gvm_settings[which].var));
-		
-		if (which == USER_MOUNT_COMMAND) {
-			/* reset the mount_commands default mount program */
-			mount_commands[0].command = config.mount_command;
-			g_free (mount_commands[0].path);
-			mount_commands[0].path = NULL;
-		} else if (which == USER_UNMOUNT_COMMAND) {
-			/* reset the unmount_commands default unmount program */
-			unmount_commands[0].command = config.unmount_command;
-			g_free (unmount_commands[0].path);
-			unmount_commands[0].path = NULL;
-		}
 	} else if (gvm_settings[which].type == TYPE_BOOL) {
 		*((int *) gvm_settings[which].var) = gconf_value_get_bool (value);
 		dbg ("setting changed: bool: %s = %d\n", strrchr (gvm_settings[which].key, '/') + 1,
@@ -1280,110 +1235,77 @@ gvm_device_mounted (const char *udi)
  * @return TRUE iff the mount was succesful
  */
 static gboolean
-gvm_device_mount (const char *udi, const char *device, const char *mount_point, int apply_policy)
+gvm_device_mount (const char *udi, int apply_policy)
 {
-	const char *mount_command = NULL;
 	struct _MountPolicy *policy;
-	char **argv;
-	int argc;
-	size_t i;
+	DBusMessage *dmesg, *reply;
+	DBusError error;
+	char *nil = "";
 	
-	for (i = 0; i < G_N_ELEMENTS (mount_commands); i++) {
-		if (!mount_commands[i].command || mount_commands[i].command[0] == '\0')
-			continue;
-		
-		if (!mount_commands[i].path) {
-			if (g_shell_parse_argv (mount_commands[i].command, &argc, &argv, NULL)) {
-				if (argv[0][0] != '/') {
-					if (!(mount_commands[i].path = g_find_program_in_path (argv[0]))) {
-						/* program doesn't exist or not in path */
-						mount_commands[i].command = NULL;
-						continue;
-					}
-				} else
-					mount_commands[i].path = g_strdup (argv[0]);
-				g_strfreev (argv);
-			} else {
-				/* command-line is busted */
-				mount_commands[i].command = NULL;
-				continue;
-			}
-		}
-		
-		if (access (mount_commands[i].path, F_OK | X_OK) == 0) {
-			mount_command = mount_commands[i].command;
-			goto mount;
-		}
+	if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
+						    "org.freedesktop.Hal.Device.Volume",
+						    "Mount")))
+		return FALSE;
+	
+	if (!(dbus_message_append_args (dmesg, DBUS_TYPE_STRING, &nil, DBUS_TYPE_STRING, &nil, DBUS_TYPE_INVALID))) {
+		dbus_message_unref (dmesg);
+		return FALSE;
 	}
 	
-	return FALSE;
-	
- mount:
-	
 	policy = g_new (struct _MountPolicy, 1);
 	policy->udi = g_strdup (udi);
 	policy->apply = apply_policy;
 	
 	g_hash_table_insert (mount_table, policy->udi, policy);
 	
-	dbg ("mounting %s...\n", udi);
-	if (gvm_run_command (mount_command, udi, device, mount_point))
-		return TRUE;
+	dbus_error_init (&error);
+	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+		dbg ("mount failed for %s: %s\n", udi, error.message);
+		g_hash_table_remove (mount_table, policy->udi);
+		dbus_message_unref (dmesg);
+		dbus_error_free (&error);
+		g_free (policy->udi);
+		g_free (policy);
+		return FALSE;
+	}
 	
-	dbg ("mount failed: %s\n", udi);
-	g_hash_table_remove (mount_table, policy->udi);
-	g_free (policy->udi);
-	g_free (policy);
+	dbus_message_unref (dmesg);
+	dbus_message_unref (reply);
 	
-	return FALSE;
+	return TRUE;
 }
 
+
 /*
  * gvm_device_unmount - unmount the given device.
  *
  * @return TRUE iff the unmount was succesful
  */
 static gboolean
-gvm_device_unmount (const char *udi, const char *device, const char *mount_point)
+gvm_device_unmount (const char *udi)
 {
-	const char *unmount_command = NULL;
-	char **argv;
-	int argc;
-	size_t i;
+	DBusMessage *dmesg, *reply;
+	DBusError error;
 	
-	for (i = 0; i < G_N_ELEMENTS (unmount_commands); i++) {
-		if (!unmount_commands[i].command || unmount_commands[i].command[0] == '\0')
-			continue;
-		
-		if (!unmount_commands[i].path) {
-			if (g_shell_parse_argv (unmount_commands[i].command, &argc, &argv, NULL)) {
-				if (argv[0][0] != '/') {
-					if (!(unmount_commands[i].path = g_find_program_in_path (argv[0]))) {
-						/* command doesn't exist or not in path */
-						unmount_commands[i].command = NULL;
-						continue;
-					}
-				} else
-					unmount_commands[i].path = g_strdup (argv[0]);
-				g_strfreev (argv);
-			} else {
-				/* command-line is busted */
-				unmount_commands[i].command = NULL;
-				continue;
-			}
-		}
-		
-		if (access (unmount_commands[i].path, F_OK | X_OK) == 0) {
-			unmount_command = unmount_commands[i].command;
-			goto unmount;
-		}
+	dbg ("unmounting %s...\n", udi);
+	
+	if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
+						    "org.freedesktop.Hal.Device.Volume",
+						    "UnMount")))
+		return FALSE;
+	
+	dbus_error_init (&error);
+	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+		dbg ("unmount failed for %s: %s\n", udi, error.message);
+		dbus_message_unref (dmesg);
+		dbus_error_free (&error);
+		return FALSE;
 	}
 	
-	return FALSE;
+	dbus_message_unref (dmesg);
+	dbus_message_unref (reply);
 	
- unmount:
-	
-	return gvm_run_command (unmount_command, udi, device, mount_point);
+	return TRUE;
 }
 
 /*
@@ -1403,7 +1325,7 @@ cda_extra_cb (GvmPromptCtx *ctx, int action)
 	switch (action) {
 	case GVM_RESPONSE_BROWSE:
 		if (!gvm_udi_is_subfs_mount (ctx->udi))
-			gvm_device_mount (ctx->udi, ctx->device, NULL, TRUE);
+			gvm_device_mount (ctx->udi, TRUE);
 		break;
 	case GVM_RESPONSE_PLAY:
 		gvm_run_cdplayer (ctx->udi, ctx->device, NULL);
@@ -1438,7 +1360,7 @@ gvm_ask_mixed (const char *udi)
 		gvm_prompt (ctx, 0, NULL);
 	} else if (config.automount_media) {
 		if (!gvm_udi_is_subfs_mount (udi))
-			gvm_device_mount (udi, device, NULL, TRUE);
+			gvm_device_mount (udi, TRUE);
 	} else if (config.autoplay_cda) {
 		gvm_run_cdplayer (udi, device, NULL);
 	}
@@ -1573,7 +1495,7 @@ gvm_cdrom_policy (const char *udi)
 		gvm_ask_mixed (udi);
 	} else if (has_data) {
 		if (config.automount_media && !gvm_udi_is_subfs_mount (udi))
-			gvm_device_mount (udi, device, NULL, TRUE);
+			gvm_device_mount (udi, TRUE);
 	} else if (is_blank) {
 		if ((type = gvm_cdrom_media_is_writable (udi)))
 			gvm_run_cdburner (udi, type, device, NULL);
@@ -1731,7 +1653,7 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 	if (config.automount_drives && mountable) {
 		if (!gvm_udi_is_subfs_mount (udi)) {
 			if (gvm_automount_enabled (udi)) {
-				gvm_device_mount (udi, device, NULL, TRUE);
+				gvm_device_mount (udi, TRUE);
 			} else {
 				dbg ("storage.automount_enabled_hint set to false on %s, not mounting\n", udi);
 			}
@@ -1989,7 +1911,7 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 		/* unmount all volumes mounted during lifetime */
 		
 		for (l = mounted_volumes; l != NULL; l = n) {
-			n = g_slist_next (l);
+			n = l->next;
 			if (strcmp (udi, (const char *) l->data) == 0) {
 				g_free (l->data);
 				mounted_volumes = g_slist_delete_link (mounted_volumes, l);
@@ -2008,11 +1930,68 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
  *  @param  message             D-BUS message with parameters
  */
 static void
-hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
-		      const char *udi __attribute__((__unused__)), 
-		      const char *condition_name __attribute__((__unused__)),
-		      const char *condition_detail __attribute__((__unused__)))
+hal_device_condition (LibHalContext *ctx, const char *udi, const char *condition_name,
+		      const char *condition_details __attribute__((__unused__)))
 {
+	if (!strcmp (condition_name, "EjectPressed")) {
+		char *storage, **volumes = NULL;
+		DBusMessage *dmesg, *reply;
+		const char *volume = udi;
+		DBusError error;
+		int i, n;
+		
+		if (!libhal_device_get_property_bool (ctx, udi, "block.is_volume", NULL)) {
+			dbus_error_init (&error);
+			volumes = libhal_find_device_by_capability (ctx, "volume", &n, &error);
+			if (dbus_error_is_set (&error)) {
+				dbus_error_free (&error);
+				return;
+			}
+			
+			volume = NULL;
+			for (i = 0; i < n; i++) {
+				volume = volumes[i];
+				
+				if (!(storage = libhal_device_get_property_string (ctx, volume, "info.parent", NULL)))
+					continue;
+				
+				if (!strcmp (udi, storage)) {
+					libhal_free_string (storage);
+					break;
+				}
+				
+				libhal_free_string (storage);
+			}
+		}
+		
+		dbg ("ejecting %s...\n", volume);
+		
+		if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", volume,
+							    "org.freedesktop.Hal.Device.Volume",
+							    "Eject"))) {
+			if (volumes)
+				libhal_free_string_array (volumes);
+			return;
+		}
+		
+		dbus_error_init (&error);
+		if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+			dbg ("eject failed for %s: %s\n", volume, error.message);
+			dbus_message_unref (dmesg);
+			dbus_error_free (&error);
+			
+			if (volumes)
+				libhal_free_string_array (volumes);
+			
+			return;
+		}
+		
+		if (volumes)
+			libhal_free_string_array (volumes);
+		
+		dbus_message_unref (dmesg);
+		dbus_message_unref (reply);
+	}
 }
 
 /** Integrate a dbus mainloop. 
@@ -2025,15 +2004,13 @@ hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
 static dbus_bool_t 
 hal_mainloop_integration (LibHalContext *ctx, DBusError *error)
 {
-	DBusConnection *dbus_connection;
-
 	dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, error);
-
+	
 	if (dbus_error_is_set (error))
 		return FALSE;
 	
         dbus_connection_setup_with_g_main (dbus_connection, NULL);
-
+	
 	libhal_ctx_set_dbus_connection (ctx, dbus_connection);
 	
 	return TRUE;
@@ -2077,14 +2054,14 @@ gvm_do_hal_init (void)
 		libhal_ctx_free (ctx);
 		return NULL;
 	}
-
+	
 	if (!libhal_ctx_init (ctx, &error)) {
 		warn ("hal_initialize failed: %s", error.message);
 		dbus_error_free (&error);
 		libhal_ctx_free (ctx);
 		return NULL;
 	}
-
+	
 	/*
 	 * Do something to ping the HAL daemon - the above functions will
 	 * succeed even if hald is not running, so long as DBUS is.  But we
@@ -2101,7 +2078,7 @@ gvm_do_hal_init (void)
 	}
 	
 	libhal_free_string_array (devices);
-
+	
 	return ctx;
 }
 
@@ -2170,7 +2147,7 @@ mount_all (LibHalContext *ctx)
 		/* mount the device */
 		if ((dev = libhal_device_get_property_string (ctx, udi, "block.device", &error))) {
 			dbg ("mount_all: mounting %s\n", dev);
-			gvm_device_mount (udi, dev, NULL, FALSE);
+			gvm_device_mount (udi, FALSE);
 			libhal_free_string (dev);
 		} else {
 			warn ("mount_all: no device for udi=%s: %s", udi, error.message);
@@ -2188,26 +2165,17 @@ mount_all (LibHalContext *ctx)
  *  @param  ctx                 LibHal context
  */
 static void
-unmount_all (LibHalContext *ctx)
+unmount_all (void)
 {
-	char *udi, *device;
-	DBusError error;
 	GSList *l;
 	
-	dbg ("unmounting all volumes that we saw mounted in our life\n");
-	dbus_error_init (&error);
-	for (l = mounted_volumes; l != NULL; l = g_slist_next (l)) {
-		udi = l->data;
+	dbg ("unmounting all volumes that we mounted in our lifetime\n");
+	
+	for (l = mounted_volumes; l != NULL; l = l->next) {
+		const char *udi = l->data;
 		
-		if ((device = libhal_device_get_property_string (ctx, udi, "block.device", &error))) {
-			dbg ("unmount_all: unmounting %s\n", device);
-			gvm_device_unmount (udi, device, NULL);
-			libhal_free_string (device);
-		} else {
-			warn ("unmount_all: no device for udi=%s: %s", udi, error.message);
-			if (dbus_error_is_set (&error))
-				dbus_error_free (&error);
-		}
+		dbg ("unmount_all: unmounting %s...\n", udi);
+		gvm_device_unmount (udi);
 	}
 }
 
@@ -2247,7 +2215,7 @@ sigterm_iochn_data (GIOChannel *source,
 	
 	dbg ("Received SIGTERM, initiating shutdown\n");
 	
-	unmount_all (hal_ctx);
+	unmount_all ();
 	
 	gtk_main_quit ();
 	
@@ -2261,7 +2229,7 @@ gvm_die (GnomeClient *client __attribute__((__unused__)),
 {
 	dbg ("Received 'die', initiating shutdown\n");
 	
-	unmount_all (hal_ctx);
+	unmount_all ();
 	
 	gtk_main_quit ();
 }
